#ifndef __UTILS_STRING_H__
#define __UTILS_STRING_H__

// character converstion macros
#define LOWER(c)		((c) >= 'A' && (c) <= 'Z' ? (c)+'a'-'A' : (c))
#define UPPER(c)		((c) >= 'a' && (c) <= 'z' ? (c)+'A'-'a' : (c))

#define NullString String("")
#include <string>

#ifdef QT_DLL
#include <qstring.h>
#endif

class String //: public std::string
{
public:
//	String ( String str ) { m_s = str.c_str(); }
//	String ( const String &str ) { m_s = str.c_str(); }
	String ( const std::string &str ) { m_s = str; }
	String ( const char *str ) { m_s = std::string(str); }
	String ( const long str ) { FromInt ( str ); }
	String ( const float str ) { FromFloat ( str ); }
	String ( const char str ) { m_s = str; }
#ifdef QT_DLL
	String ( QString &str ) { if ( str.isEmpty() ) m_s = ""; else m_s = str.data(); }
	String ( const QString &str ) { if ( str.isEmpty() ) m_s = ""; else m_s = str.data(); }
#endif
	String () {}

	~String () {}

#ifdef QT_DLL
	String operator= ( QString str ) 
	{ 
		if ( str.isEmpty() ) 
			m_s = ""; 
		else 
			m_s = str.data();
		return (*this); 
	}
	String &operator= ( const QString &str ) { if ( str.isEmpty() ) m_s = ""; else m_s = str.data(); return (*this); }
#endif
	String &operator= ( const std::string &str ) { m_s = str; return (*this); }
	String &operator= ( const char *str ) { m_s = std::string(str); return (*this); }
	String &operator= ( String str ) { m_s = str.ToString(); return (*this); }
	String &operator= ( const long str ) { FromInt ( str ); return (*this); }
	String &operator= ( const float str ) { FromFloat ( str ); return (*this); }
	String &operator= ( const char str ) { m_s = str; return (*this); }

#ifdef QT_DLL
	String operator+ ( QString &str ) { return String(m_s + str.data()); }
	String operator+ ( const QString &str ) { return String(m_s + str.data()); }
#endif
	String operator+ ( String str ) { return String(m_s + str.ToString()); }
	String operator+ ( const char *str ) { return String(m_s + std::string(str)); }
	String operator+ ( const std::string &str ) { return String(m_s + str); }
	String operator+ ( const char str ) { return String(m_s + str); }
	String operator+ ( const long  str ) { String s(str); s = m_s + s.ToString(); return s; }
	String operator+ ( const float str ) { String s(str); s = m_s + s.ToString(); return s; }

	String &operator+= ( String str ) { m_s += str.ToString(); return (*this); }
	String &operator+= ( const char *str ) { m_s += std::string(str); return (*this); }
	String &operator+= ( const std::string &str ) { m_s += str; return (*this); }
	String &operator+= ( const char str ) { m_s += str; return (*this); }
	String &operator+= ( const long  str ) { String s(str); m_s += s.ToString(); return (*this); }
	String &operator+= ( const float str ) { String s(str); m_s += s.ToString(); return (*this); }

	const char operator[] ( int num ) { return m_s[num]; }

#ifdef QT_DLL
	bool operator== ( QString &str ) 
	{ 
		return (m_s == str.data());
	}
#endif
	bool operator!= ( String str )
	{ 
		if ( str.Length() != m_s.length() ) return true;
		for ( int i = 0; i < str.Length(); i++ )
			if ( LOWER(m_s[i]) != LOWER(str[i]) )
				return true;
		return false; 
	}
	bool operator!= ( const char *str ) 
	{ 
		if ( strlen(str) != m_s.length() ) return true;
		
		for ( int i = 0; i < m_s.length(); i++ )
			if ( LOWER(m_s[i]) != LOWER(*(str + i)) )
				return true;
		return false; 
	}
	bool operator== ( String str ) 
	{ 
		if ( str.Length() != m_s.length() ) return false;
		for ( int i = 0; i < str.Length(); i++ )
			if ( LOWER(m_s[i]) != LOWER(str[i]) )
				return false;
		return true; 
	}
	bool operator== ( const std::string &str ) 
	{ 
		if ( str.length() != m_s.length() ) return false;
		for ( int i = 0; i < str.length(); i++ )
			if ( LOWER(m_s[i]) != LOWER(str[i]) )
				return false;
		return true; 
	}
	bool operator== ( const char *str ) 
	{ 
		if ( strlen(str) != m_s.length() ) return false;
		
		for ( int i = 0; i < m_s.length(); i++ )
			if ( LOWER(m_s[i]) != LOWER(*(str + i)) )
				return false;
		return true; 
	}

	bool Compare ( String &str )
	{
		if ( str.Length() != m_s.length() ) return false;
		for ( int i = 0; i < str.Length(); i++ )
			if ( m_s[i] != str[i] )
				return false;
		return true; 
	}
	bool Compare ( const char *str ) 
	{ 
		if ( strlen(str) != m_s.length() ) return false;
		
		for ( int i = 0; i < m_s.length(); i++ )
			if ( m_s[i] != *(str + i) )
				return false;
		return true; 
	}

	const char *ToChar  () { return m_s.c_str(); }
	const int   ToInt   () { return atoi(m_s.c_str()); }
	const long  ToLong  () { return atoi(m_s.c_str()); }
	const float ToFloat () { return atof(m_s.c_str()); }
	std::string &ToString () { return m_s; }
#ifdef QT_DLL
	QString   ToQString() { return QString(m_s.c_str()); }
#endif

	const char *c_str   () { return m_s.c_str(); }

	bool CharIsNum ( int c )
	{
		if ( (m_s[c] >= '0') && (m_s[c] <= '9') )
			return true;
		return false;
	}

	static String Number ( const long num ) { char c[100]; sprintf ( c, "%ld", num ); return std::string(c); }
	String &FromChar   ( const char  *str ) { m_s = std::string(str); return (*this); }
	String &FromInt    ( const long   str ) { char c[100]; sprintf ( c, "%ld", str ); m_s = std::string(c); return (*this); }
	String &FromFloat  ( const float  str, int dp = -1 ) 
	{ 
		char c[100]; 
		if ( dp != -1 )
		{
			switch ( dp )
			{
				case 0:
					sprintf ( c, "%.0f", str ); 
					break;
				case 1:
					sprintf ( c, "%.1f", str ); 
					break;
				case 2:
					sprintf ( c, "%.2f", str ); 
					break;
				case 3:
					sprintf ( c, "%.3f", str ); 
					break;
				case 4:
					sprintf ( c, "%.4f", str ); 
					break;
				case 5:
					sprintf ( c, "%.5f", str ); 
					break;
				case 6:
					sprintf ( c, "%.6f", str ); 
					break;

				default:
					sprintf ( c, "%.7f", str ); 
			}
		}
		else
			sprintf ( c, "%f", str ); 

		m_s = std::string(c); 
		return (*this); 
	}

	const int  Length() { return m_s.length(); }
	const bool Empty () { return m_s.empty();  }

	String &ToLower () 
	{
		for ( int i = 0; i < m_s.length(); i++ )
			m_s[i] = LOWER(m_s[i]);
		return (*this);
	}
	String &ToLower ( const char *str )
	{
		m_s = "";
		char *c = (char *)str;
		while ( *c != '\0' )
		{
			m_s += LOWER(*c);
			++c;
		}
		return (*this);
	}

	String &ToUpper () 
	{
		for ( int i = 0; i < m_s.length(); i++ )
			m_s[i] = UPPER(m_s[i]);
		return (*this);
	}
	String &ToUpper ( const char *str )
	{
		m_s = "";
		char *c = (char *)str;
		while ( *c != '\0' )
		{
			m_s += UPPER(*c);
			++c;
		}
		return (*this);
	}


	int NumToken ( char token )
	{
		// finds the number of tokens in a string
		int found = 0;

		std::string tmpstr = m_s;

		// ignore the initial spaces
		std::string::size_type pos = m_s.find_first_not_of(token, 0);

		// remove end spaces
		int i = tmpstr.size() - 1;
		while ( tmpstr[i] == ' ' )
			i--;
		tmpstr.erase ( i + 1, tmpstr.size() - i );

		// count tokens
		while ( pos < std::string::npos )
		{
			pos = tmpstr.find (token,pos + 1);
			while (tmpstr[pos] == ' ')
				pos++;
			found++;
		}

		// return number of tokens
		return found;
	}
	String GetToken ( int num, char token )
	{
		std::string newstr;

		int whichtok = 1;
		// get first token
		std::string::size_type lastPos = m_s.find_first_not_of(token, 0);
		std::string::size_type pos     = m_s.find_first_of(token, lastPos);


		while (std::string::npos != pos || std::string::npos != lastPos)
		{
			if ( whichtok == num )
			{
				newstr = m_s.substr(lastPos, pos - lastPos);
				break;
			}
			// Found a token, add it to the vector.
			whichtok++;
			// Skip delimiters.  Note the "not_of"
			lastPos = m_s.find_first_not_of(token, pos);
			// Find next "non-delimiter"
			pos = m_s.find_first_of(token, lastPos);

		}
		return String(newstr);
	}
	String GetToken ( int from, int to, char token )
	{
		std::string newstr;

		int whichtok = 1;
		// get first token
		std::string::size_type lastPos = m_s.find_first_not_of(token, 0);
		std::string::size_type pos     = m_s.find_first_of(token, lastPos);


		while (std::string::npos != pos || std::string::npos != lastPos)
		{
			if ( to == -1 )
			{
				if ( whichtok >= from )
				{
					if ( newstr != "" ) newstr = newstr + token;
					newstr = newstr + m_s.substr(lastPos, pos - lastPos);
				}
			}
			else
			{
				if ( (whichtok >= from) && (whichtok <= to) )
				{
					if ( newstr != "" ) newstr = newstr + token;
					newstr = newstr + m_s.substr(lastPos, pos - lastPos);
				}
				if ( whichtok > to )
					break;
			}
			// Found a token, add it to the vector.
			whichtok++;
			// Skip delimiters.  Note the "not_of"
			lastPos = m_s.find_first_not_of(token, pos);
			// Find next "non-delimiter"
			pos = m_s.find_first_of(token, lastPos);

		}
		return String(newstr);
	}

	int FindToken ( String &str, char token )
	{
		// get first token
		std::string::size_type lastPos = m_s.find_first_not_of(token, 0);
		std::string::size_type pos     = m_s.find_first_of(token, lastPos);

		int tok = 1;
		while (std::string::npos != pos || std::string::npos != lastPos)
		{
			std::string newstr = m_s.substr(lastPos, pos - lastPos);
			if ( str == newstr )
				return tok;
			++tok;
		}
		return 0;
	}

	String DelToken ( int num, char token )
	{
		std::string newstr = "";

		int whichtok = 1;
		// get first token
		std::string::size_type lastPos = m_s.find_first_not_of(token, 0);
		std::string::size_type pos     = m_s.find_first_of(token, lastPos);

		while (std::string::npos != pos || std::string::npos != lastPos)
		{
			if ( whichtok != num )
			{
				if ( newstr == "" )
					newstr = m_s.substr(lastPos, pos - lastPos);
				else
					newstr += token + m_s.substr(lastPos, pos - lastPos);
			}

			// Found a token, add it to the vector.
			whichtok++;
			// Skip delimiters.  Note the "not_of"
			lastPos = m_s.find_first_not_of(token, pos);
			// Find next "non-delimiter"
			pos = m_s.find_first_of(token, lastPos);

		}
		return String(newstr);
	}
	String DelToken ( String &find, char token, bool single = true )
	{
		std::string newstr = "";

		// get first token
		std::string::size_type lastPos = m_s.find_first_not_of(token, 0);
		std::string::size_type pos     = m_s.find_first_of(token, lastPos);

		while (std::string::npos != pos || std::string::npos != lastPos)
		{
			std::string s = m_s.substr(lastPos, pos - lastPos);
			if ( find != s.c_str() )
			{
				if ( newstr == "" )
					newstr = s;
				else
					newstr += token + s;
			}
			else if ( single )
			{
				lastPos = m_s.find_first_not_of(token, pos);
				if ( newstr == "" )
					newstr = m_s.substr(lastPos, m_s.length() - lastPos);
				else
					newstr += token + m_s.substr(lastPos, m_s.length() - lastPos);
				return String(newstr);
			}

			// Skip delimiters.  Note the "not_of"
			lastPos = m_s.find_first_not_of(token, pos);
			// Find next "non-delimiter"
			pos = m_s.find_first_of(token, lastPos);

		}
		return String(newstr);
	}

	String &Addtoken ( char token, String &t )
	{
		if ( !m_s.empty() )
			m_s += token;
		m_s += t.ToString();

		return (*this);
	}
	String &AddToken ( char token, String &t )
	{
		if ( !m_s.empty() )
			m_s += token;
		m_s += t.ToString();

		return (*this);
	}
	String &AddToken ( char token, char *t )
	{
		if ( !m_s.empty() )
			m_s += token;
		m_s += t;

		return (*this);
	}
	/*
	bool IsToken (const std::string &str, const std::string &tokens, char token )
	{
		int jmax = NumToken ( tokens, token );

		// check against each token
		for ( int j = 1; j <= jmax; j++ )
		{
			if ( str == GetToken ( tokens, j, token ) )
				return true;
		}
		return false;
	}*/

	int FindPos ( String str, int start = 0 )
	{
		int pos = 0;
		for ( int i = start; i < m_s.length(); i++ )
		{
			if ( m_s[i] == str[pos] )
			{
				++pos;
				if ( pos >= str.Length() )
					return i - (str.Length() - 1);
			}
			else
				pos = 0;
		}

		return -1;
	}

	bool IsIn ( const char *str ) { return IsIn ( String(str) ); }
	bool IsIn ( String str )
	{
		int check = 0;
		for ( int i = 0; i < m_s.length(); i++ )
		{
			if ( LOWER(m_s[i]) == LOWER(str[check]) )
				++check;
			else
				check = 0;

			if ( check >= str.Length() )
				return true;
		}
		return false;
	}
	bool IsIn ( char c )
	{
		bool f = false;
		for ( int i = 0; i < m_s.length(); i++ )
		{
			if ( m_s[i] == c )
			{
				f = true;
				break;
			}
		}
		return f;
	}
	bool IsInExact ( const char *str ) { return IsInExact ( String(str) ); }
	bool IsInExact ( String str )
	{
		int check = 0;
		for ( int i = 0; i < m_s.length(); i++ )
		{
			if ( m_s[i] == str[check] )
				++check;
			else
				check = 0;

			if ( check >= str.Length() )
				return true;
		}
		return false;
	}

	String *SplitToken ( char token, int *max )
	{
		*max = NumToken ( token );

		String *newstr = new String[*max];

		int whichtok = 0;
		// get first token
		std::string::size_type lastPos = m_s.find_first_not_of(token, 0);
		std::string::size_type pos     = m_s.find_first_of(token, lastPos);


		while (std::string::npos != pos || std::string::npos != lastPos)
		{
			// set token
			newstr[whichtok] = m_s.substr(lastPos, pos - lastPos);

			// move to next token
			whichtok++;
			if ( whichtok >= *max )
				break;

			// Skip delimiters.  Note the "not_of"
			lastPos = m_s.find_first_not_of(token, pos);
			// Find next "non-delimiter"
			pos = m_s.find_first_of(token, lastPos);
		}

		return newstr;
	}

	String Remove ( char c )
	{
		std::string newstr;
		for ( int i = 0; i < m_s.length(); i++ )
		{
			if ( m_s[i] != c )
				newstr += m_s[i];
		}
		return newstr;
	}

	void RemoveChar ( char c )
	{
		std::string old = m_s;
		m_s = "";
		for ( int i = 0; i < old.length(); i++ )
		{
			if ( old[i] != c )
				m_s += old[i];
		}
	}

	void RemoveChar ( char *c )
	{
		std::string old = m_s;
		m_s = "";
		for ( int i = 0; i < old.length(); i++ )
		{
			bool found = false;
			char *nc = &c[0];
			while ( *nc != '\0' )
			{
				if ( old[i] == *nc )
				{
					found = true;
					break;
				}
				++nc;
			}
			if ( !found )
				m_s += old[i];
		}
	}

	String Left ( int num )
	{
		if ( num < 0 )
			num = m_s.length() + num;
		if ( num > m_s.length() )
			num = m_s.length();

		char *str = new char[num + 1];
		int i;
		for ( i = 0; i < num; i++ )
			str[i] = m_s[i];
		str[i] = '\0';
		return String(str);
	}

	String Right ( int num )
	{
		if ( num < 0 )
			num = m_s.length() + num;
		if ( num > m_s.length() )
			num = m_s.length();
		int start = m_s.length() - num, i;
		char *str = new char[num + 1];
		for ( i = start; i < m_s.length(); i++ )
			str[i - start] = m_s[i];
		str[i - start] = '\0';
		return String(str);
	}
	String Mid ( int start, int len )
	{
		int end = start + len;
		if ( end >= m_s.length() )
		{
			end = m_s.length();
			len = end - start;
		}

		char *str = new char[len + 1];
		int i;
		for ( i = start; i < end; i++ )
			str[i - start] = m_s[i - 1];
		str[i - start] = '\0';
		return String(str);
	}

	String &Erase ( int start, int num ) { m_s.erase ( start, num ); return (*this); }

	String FindReplace ( const char *find, const char *replace ) { return FindReplace ( String(find), String(replace) ); }
	String FindReplace ( String find, const char *replace ) { return FindReplace ( find, String(replace) ); }
	String FindReplace ( const char *find, String replace ) { return FindReplace ( String(find), replace ); }
	String FindReplace ( String find, String replace )
	{
		std::string::size_type pos = m_s.find ( find.ToString(), 0 );
		while ( pos < std::string::npos )
		{
			m_s.replace ( pos, find.Length(), replace.ToString() );
			pos = m_s.find ( find.ToString(), pos );
		}

		return m_s;
	}

private:
	std::string m_s;
};

#endif //__UTILS_STRING_H__
